import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import seaborn as sns


class Plotter:
    @staticmethod
    def general_ohlcv_plot(ax: plt.Axes, ohlcv: pd.DataFrame) -> plt.Axes:
        """
        Plot the general OHLCV chart.

        Args:
            ax (plt.Axes): The axes to plot the chart on.
            ohlcv (pd.DataFrame): The OHLCV data.

        Returns:
            plt.Axes: The axes with the plotted chart.
        """
        ax.plot(ohlcv.index, ohlcv["close"], color="black", alpha=0.1)
        ax.fill_between(ohlcv.index, ohlcv["high"], ohlcv["low"], alpha=1)
        ax.scatter(
            ohlcv.index,
            ohlcv["close"],
            color="black",
            s=(ohlcv["volume"] - ohlcv["volume"].min())
            / (ohlcv["volume"].max() - ohlcv["volume"].min())
            * 300,
            alpha=0.3,
        )
        ax.set_xlabel("Date")
        ax.set_ylabel("Price")
        return ax

    @staticmethod
    def draw_regression_plot(ax: plt.Axes, ohlcv: pd.DataFrame, slice: int = 1) -> plt.Axes:
        """
        Draw a regression plot for the OHLCV data.

        Args:
            ax (plt.Axes): The axes to plot the chart on.
            ohlcv (pd.DataFrame): The OHLCV data.
            slice (int, optional): The number of slices to divide the data into. Defaults to 1.

        Returns:
            plt.Axes: The axes with the plotted chart.
        """
        window = len(ohlcv) // slice
        for idx in range(slice):
            n_volume_series = (
                (ohlcv["volume"] - ohlcv["volume"].min())
                / (ohlcv["volume"].max() - ohlcv["volume"].min())
                * 300
            )
            _volume = n_volume_series.iloc[idx * window : window * (idx + 1)]
            _ohlcv = ohlcv.iloc[idx * window : window * (idx + 1)]
            sns.regplot(
                x=mdates.date2num(_ohlcv.index),
                y=_ohlcv["close"],
                ax=ax,
                scatter_kws={
                    "s": _volume,
                    "alpha": 0.5,
                },
            )
        plt.gca().xaxis.set_major_formatter(mdates.DateFormatter("%Y-%m-%d"))
        return ax



from typing import Union
from datetime import datetime
import pandas as pd

from .analyser import CoefficientAnalyser, ProfitAnalyser, VarianceAnalyser
from .utils import filter_date


class MultiOhlcvAnalyser:
    def __init__(self, multi_ohlcv: pd.DataFrame) -> None:
        """
        Initialize the MultiOhlcvAnalyser class.

        Parameters:
        - multi_ohlcv (pd.DataFrame): The multi-ohlcv data.

        Returns:
        None
        """
        self.ohlcv = multi_ohlcv.copy()

    @staticmethod
    def _get_groupby_code(ohlcv: pd.DataFrame, start: str, end: str):
        """
        Get the groupby object of the ohlcv data based on the code column.

        Parameters:
        - ohlcv (pd.DataFrame): The ohlcv data.
        - start (str): The start date.
        - end (str): The end date.

        Returns:
        pd.core.groupby.generic.DataFrameGroupBy: The groupby object.
        """
        groupby_ohlcv = filter_date(ohlcv, start, end).groupby("code")
        return groupby_ohlcv

    def info(
        self,
        start: Union[str, datetime] = None,
        end: Union[str, datetime] = None,
    ) -> pd.DataFrame:
        """
        Get general information about the multi-ohlcv data.

        Parameters:
        - start (Union[str, datetime]): The start date. Default is None.
        - end (Union[str, datetime]): The end date. Default is None.

        Returns:
        pd.DataFrame: The information dataframe.
        """
        ohlcv = filter_date(self.ohlcv, start, end)
        groupby_code = self._get_groupby_code(ohlcv, start, end)
        profit_row = groupby_code.apply(
            lambda x: ProfitAnalyser.get_start_end_profit(x["close"])
        )
        info_dict = {
            "total_stock_code": len(set(ohlcv["code"])),
            "start_date": ohlcv.index.min(),
            "end_date": ohlcv.index.max(),
            "market_average_profit": (profit_row.mean().round(2)),
            "increased_stock_pct": round(
                len(profit_row[profit_row > 0]) / len(profit_row), 2
            )
            * 100,
            "decreased_stock_pct": round(
                len(profit_row[profit_row <= 0]) / len(profit_row), 2
            )
            * 100,
        }

        info_df = pd.DataFrame.from_dict(
            info_dict, orient="index", columns=["value"]
        )
        return info_df

    def coef(
        self,
        arg: str,
        start: Union[str, datetime] = None,
        end: Union[str, datetime] = None,
    ) -> pd.Series:
        """
        Get the coefficient series for a given argument.

        Parameters:
        - arg (str): The argument to calculate the coefficient.
        - start (Union[str, datetime]): The start date. Default is None.
        - end (Union[str, datetime]): The end date. Default is None.

        Returns:
        pd.Series: The coefficient series.
        """
        groupby_code = self._get_groupby_code(self.ohlcv, start, end)
        coef_series = groupby_code.apply(
            lambda x: CoefficientAnalyser.get_coefficient(x[arg])
        ).rename(f"{arg}_coef")
        return coef_series

    def normalized_coef(
        self,
        arg: str,
        start: Union[str, datetime] = None,
        end: Union[str, datetime] = None,
    ) -> pd.Series:
        """
        Get the normalized coefficient series for a given argument.

        Parameters:
        - arg (str): The argument to calculate the normalized coefficient.
        - start (Union[str, datetime]): The start date. Default is None.
        - end (Union[str, datetime]): The end date. Default is None.

        Returns:
        pd.Series: The normalized coefficient series.
        """
        groupby_code = self._get_groupby_code(self.ohlcv, start, end)
        normalized_coef_series = groupby_code.apply(
            lambda x: CoefficientAnalyser.get_normalized_coefficient(x[arg])
        ).rename(f"{arg}_normalized_coef")
        return normalized_coef_series

    def coef_score(
        self,
        arg: str,
        start: Union[str, datetime] = None,
        end: Union[str, datetime] = None,
    ) -> pd.Series:
        """
        Get the coefficient score series for a given argument.

        Parameters:
        - arg (str): The argument to calculate the coefficient score.
        - start (Union[str, datetime]): The start date. Default is None.
        - end (Union[str, datetime]): The end date. Default is None.

        Returns:
        pd.Series: The coefficient score series.
        """
        groupby_code = self._get_groupby_code(self.ohlcv, start, end)
        coef_score_series = groupby_code.apply(
            lambda x: CoefficientAnalyser.get_coefficient_score(x[arg])
        ).rename(f"{arg}_coef_score")
        return coef_score_series

    def oc_variance(
        self,
        start: Union[str, datetime] = None,
        end: Union[str, datetime] = None,
    ) -> pd.Series:
        """
        Get the (open - close) variance series.

        Parameters:
        - start (Union[str, datetime]): The start date. Default is None.
        - end (Union[str, datetime]): The end date. Default is None.

        Returns:
        pd.Series: The (open - close) variance series.
        """
        groupby_code = self._get_groupby_code(self.ohlcv, start, end)
        oc_variance_series = groupby_code.apply(
            lambda x: VarianceAnalyser.get_normalized_diff_variance(
                x["open"], x["close"]
            )
        ).rename("oc_variance")
        return oc_variance_series

    def hl_variance(
        self,
        start: Union[str, datetime] = None,
        end: Union[str, datetime] = None,
    ) -> pd.Series:
        """
        Get the (high - low) variance series.

        Parameters:
        - start (Union[str, datetime]): The start date. Default is None.
        - end (Union[str, datetime]): The end date. Default is None.

        Returns:
        pd.Series: The (high - low) variance series.
        """
        groupby_code = self._get_groupby_code(self.ohlcv, start, end)
        hl_variance_series = groupby_code.apply(
            lambda x: VarianceAnalyser.get_normalized_diff_variance(
                x["high"], x["low"]
            )
        ).rename("hl_variance")
        return hl_variance_series

    def profit(
        self,
        arg: str = "close",
        start: Union[str, datetime] = None,
        end: Union[str, datetime] = None,
    ) -> pd.Series:
        """
        Get the profit series for a given argument.

        Parameters:
        - arg (str): The argument to calculate the profit. Default is "close".
        - start (Union[str, datetime]): The start date. Default is None.
        - end (Union[str, datetime]): The end date. Default is None.

        Returns:
        pd.Series: The profit series.
        """
        ohlcv = self.ohlcv.copy()
        groupby_code = self._get_groupby_code(ohlcv, start, end)
        profit_series = groupby_code.apply(
            lambda x: ProfitAnalyser.get_start_end_profit(x[arg])
        ).rename("profit")
        return profit_series



import pandas as pd

from .analyser import ProfitAnalyser, PriceAnalyser, CoefficientAnalyser
from .utils import filter_date

class SingleOhlcvAnalyser:
    def __init__(self, single_ohlcv: pd.DataFrame):
        """
        Initialize the SingleOhlcvAnalyser class.

        Parameters:
        - single_ohlcv (pd.DataFrame): The OHLCV data for a single stock.

        """
        self.ohlcv = single_ohlcv.copy()

    def info(self, start=None, end=None) -> pd.DataFrame:
        """
        Get information about the OHLCV data.

        Parameters:
        - start (str or None): The start date for filtering the data. Default is None.
        - end (str or None): The end date for filtering the data. Default is None.

        Returns:
        - pd.DataFrame: A DataFrame containing the information about the OHLCV data.

        """
        ohlcv = filter_date(self.ohlcv, start, end)
        info_dict = {
            "stock_code": ohlcv["code"].iloc[0],
            "start_date": ohlcv.index.min(),
            "end_date": ohlcv.index.max(),
            "start_end_profit": ProfitAnalyser.get_start_end_profit(
                ohlcv["close"]
            ),
            "start_max_profit": ProfitAnalyser.get_start_max_profit(
                ohlcv["close"]
            ),
            "start_min_profit": ProfitAnalyser.get_start_min_profit(
                ohlcv["close"]
            ),
        }
        info_df = pd.DataFrame.from_dict(
            info_dict, orient="index", columns=["value"]
        )
        return info_df


    def get_price_rank_series(self, start: str, end: str, price: float) -> pd.Series:
        """
        Get the price rank series for a given date range and price.

        Parameters:
        - start (str): The start date for filtering the data.
        - end (str): The end date for filtering the data.
        - price (float): The price for calculating the price rank.

        Returns:
        - pd.Series: A Series containing the price rank information.

        """
        filtered_ohlcv = filter_date(self.ohlcv, start, end)
        statistical_prices = PriceAnalyser.get_statistical_prices(
            filtered_ohlcv["high"],
            filtered_ohlcv["low"],
            filtered_ohlcv["volume"],
        )
        price_rank_series = pd.Series(
            {
                "start_date": start.strftime("%Y-%m-%d"),
                "date_diff": (end - start).days,
                "end_date": end.strftime("%Y-%m-%d"),
                "price": price,
                "mean_price": round(statistical_prices.mean(), 2),
                "price_rank": PriceAnalyser.get_price_rank(
                    statistical_prices, price
                ),
            }
        )
        return price_rank_series

    def get_coefficient_series(self, arg: str, start: str, end: str) -> pd.Series:
        """
        Get the coefficient series for a given argument, start date, and end date.

        Parameters:
        - arg (str): The argument for calculating the coefficient.
        - start (str): The start date for filtering the data.
        - end (str): The end date for filtering the data.

        Returns:
        - pd.Series: A Series containing the coefficient information.

        """
        filtered_ohlcv = filter_date(self.ohlcv, start, end)
        coefficient_series = pd.Series(
            {
                "start_date": start.strftime("%Y-%m-%d"),
                "date_diff": (end - start).days,
                "end_date": end.strftime("%Y-%m-%d"),
                "arg": arg,
                "coefficient": CoefficientAnalyser.get_normalized_coefficient(
                    filtered_ohlcv[arg]
                ),
            }
        )
        return coefficient_series


import numpy as np
import pandas as pd


class ProfitAnalyser:
    @staticmethod
    def calc_pct(value: float) -> float:
        """
        Calculate the percentage value of a given number.
        
        Args:
            value (float): The number to calculate the percentage of.
        
        Returns:
            float: The percentage value of the given number.
        """
        return round(value * 100, 3)

    @staticmethod
    def get_start_end_profit(price_series: pd.Series) -> float:
        """
        Calculate the profit percentage between the first and last prices in a price series.
        
        Args:
            price_series (pd.Series): The price series.
        
        Returns:
            float: The profit percentage.
        """
        buying_price = price_series.iloc[0]
        selling_price = price_series.iloc[-1]
        profit = (selling_price - buying_price) / buying_price
        return ProfitAnalyser.calc_pct(profit)

    @staticmethod
    def get_start_max_profit(price_series: pd.Series) -> float:
        """
        Calculate the maximum profit percentage between the first price and the highest price in a price series.
        
        Args:
            price_series (pd.Series): The price series.
        
        Returns:
            float: The maximum profit percentage.
        """
        buying_price = price_series.iloc[0]
        max_price = price_series.max()
        max_profit = (max_price - buying_price) / buying_price
        return ProfitAnalyser.calc_pct(max_profit)

    @staticmethod
    def get_start_min_profit(price_series: pd.Series) -> float:
        """
        Calculate the minimum profit percentage between the first price and the lowest price in a price series.
        
        Args:
            price_series (pd.Series): The price series.
        
        Returns:
            float: The minimum profit percentage.
        """
        buying_price = price_series.iloc[0]
        min_price = price_series.min()
        min_profit = (min_price - buying_price) / buying_price
        return ProfitAnalyser.calc_pct(min_profit)

    @staticmethod
    def get_statistical_price_series(high_series: pd.Series, low_series: pd.Series) -> pd.Series:
        """
        Generate a statistical price series based on the high and low series.
        
        Args:
            high_series (pd.Series): The high price series.
            low_series (pd.Series): The low price series.
        
        Returns:
            pd.Series: The generated statistical price series.
        """
        mean_series = ((high_series + low_series) / 2).rename("mean")
        var_series = ((high_series - low_series) / 4).rename("var")

        def get_value_based_on_normal_dist_assumption(mean: float, var: float) -> float:
            return np.random.normal(mean, var, 1)[0]

        statistical_price_series = pd.concat(
            [mean_series, var_series], axis=1
        ).apply(
            lambda x: get_value_based_on_normal_dist_assumption(
                x["mean"], x["var"]
            ),
            axis=1,
        )
        return statistical_price_series



import numpy as np
import pandas as pd
from sklearn.linear_model import LinearRegression

lr = LinearRegression()


class CoefficientAnalyser:
    @staticmethod
    def get_coefficient(series: pd.Series) -> float:
        """
        Calculates the coefficient of a series using linear regression.

        Args:
            series (pd.Series): The input series.

        Returns:
            float: The coefficient value.
        """
        x = np.arange(1, len(series) + 1).reshape(-1, 1)
        y = series.values.reshape(-1, 1)
        lr.fit(x, y)
        coefficient = lr.coef_[0][0]
        return coefficient

    @staticmethod
    def get_coefficient_score(series: pd.Series) -> float:
        """
        Calculates the coefficient score of a series using linear regression.

        Args:
            series (pd.Series): The input series.

        Returns:
            float: The coefficient score value.
        """
        x = np.arange(1, len(series) + 1).reshape(-1, 1)
        y = series.values.reshape(-1, 1)
        lr.fit(x, y)
        score = lr.score(x, y)
        return score

    @staticmethod
    def get_normalized_coefficient(series: pd.Series) -> float:
        """
        Calculates the normalized coefficient of a series using linear regression.

        Args:
            series (pd.Series): The input series.

        Returns:
            float: The normalized coefficient value.
        """
        normalized_series = series / series.mean()
        normalized_coefficient = CoefficientAnalyser.get_coefficient(
            normalized_series
        )
        return normalized_coefficient


def filter_date(ohlcv, start, end):
    filtered_ohlcv = ohlcv.copy()
    if start:
        filtered_ohlcv = filtered_ohlcv[start <= filtered_ohlcv.index]
    if end:
        filtered_ohlcv = filtered_ohlcv[filtered_ohlcv.index <= end]
    return filtered_ohlcv



import pandas as pd
import numpy as np


class PriceAnalyser:
    @staticmethod
    def get_normalized_series(series: pd.Series, n: int = 1) -> pd.Series:
        """
        Get the normalized series by dividing each element by the mean of the series and multiplying it by n.

        Args:
            series (pd.Series): The input series.
            n (int, optional): The multiplier. Defaults to 1.

        Returns:
            pd.Series: The normalized series.
        """
        normalized_series = (series / series.mean()) * n
        return round(normalized_series).astype(int)

    @staticmethod
    def get_general_prices(
        price_series: pd.Series, volume_series: pd.Series
    ) -> np.ndarray:
        """
        Get the general prices based on the price series and volume series.

        Args:
            price_series (pd.Series): The price series.
            volume_series (pd.Series): The volume series.

        Returns:
            np.ndarray: The general prices.
        """
        normalized_volume_series = PriceAnalyser.get_normalized_series(
            volume_series, 10
        )
        prices = np.concatenate(
            (
                price_series.apply(lambda x: [x]) * normalized_volume_series
            ).values
        )
        return prices

    @staticmethod
    def get_statistical_prices(
        high_series: pd.Series, low_series: pd.Series, volume_series: pd.Series
    ) -> np.ndarray:
        """
        Get the statistical prices based on the high series, low series, and volume series.

        Args:
            high_series (pd.Series): The high series.
            low_series (pd.Series): The low series.
            volume_series (pd.Series): The volume series.

        Returns:
            np.ndarray: The statistical prices.
        """
        normalized_volume_series = PriceAnalyser.get_normalized_series(
            volume_series, 10
        )
        mean_series = ((high_series + low_series) / 2).rename("mean")
        var_series = ((high_series - low_series) / 4).rename("var")

        def get_value_based_on_normal_dist_assumption(
            mean: float, var: float, volume: float
        ) -> np.ndarray:
            return np.random.normal(mean, var, int(volume))

        statistical_prices = np.concatenate(
            pd.concat(
                [
                    mean_series,
                    var_series,
                    normalized_volume_series,
                ],
                axis=1,
            )
            .apply(
                lambda x: get_value_based_on_normal_dist_assumption(
                    x["mean"], x["var"], x["volume"]
                ),
                axis=1,
            )
            .values
        )

        return statistical_prices

    @staticmethod
    def get_price_rank(prices: np.ndarray, price: float) -> float:
        """
        Get the price rank based on the prices and a specific price.

        Args:
            prices (np.ndarray): The array of prices.
            price (float): The specific price.

        Returns:
            float: The price rank.
        """
        sorted_prices = np.sort(np.append(prices, price))
        rank = np.where(sorted_prices == price)[0][0] + 1
        percentile = round(
            ((len(sorted_prices) - rank) / len(sorted_prices)) * 100, 2
        )
        return percentile

    @staticmethod
    def _get_pressure_indicator(
        open_price: float, close_price: float
    ) -> float:
        """
        Get the pressure indicator based on the open price and close price.

        Args:
            open_price (float): The open price.
            close_price (float): The close price.

        Returns:
            float: The pressure indicator.
        """
        pressure_indicator = (close_price - open_price) / open_price * 100 * 3
        return pressure_indicator

    @staticmethod
    def get_buying_selling_general_prices(
        open_price: float, close_price: float, volume: pd.Series
    ) -> tuple[np.ndarray, np.ndarray]:
        """
        Get the buying and selling general prices based on the open price, close price, and volume.

        Args:
            open_price (float): The open price.
            close_price (float): The close price.
            volume (pd.Series): The volume series.

        Returns:
            tuple[np.ndarray, np.ndarray]: The buying prices and selling prices.
        """
        pressure_indicator = PriceAnalyser._get_pressure_indicator(
            open_price, close_price
        )
        normalized_volume = PriceAnalyser.get_normalized_series(volume, 10)
        buying_normalized_volume = (
            normalized_volume * (100 + pressure_indicator) / 200
        )
        selling_normalized_volume = (
            normalized_volume - buying_normalized_volume
        )
        buying_prices = PriceAnalyser.get_general_prices(
            close_price, buying_normalized_volume
        )
        selling_prices = PriceAnalyser.get_general_prices(
            close_price, selling_normalized_volume
        )
        return buying_prices, selling_prices

    @staticmethod
    def get_buying_selling_statistical_prices(
        open_price: float,
        close_price: float,
        high_price: pd.Series,
        low_price: pd.Series,
        volume: pd.Series,
    ) -> tuple[np.ndarray, np.ndarray]:
        """
        Get the buying and selling statistical prices based on the open price, close price, high price, low price, and volume.

        Args:
            open_price (float): The open price.
            close_price (float): The close price.
            high_price (pd.Series): The high price series.
            low_price (pd.Series): The low price series.
            volume (pd.Series): The volume series.

        Returns:
            tuple[np.ndarray, np.ndarray]: The buying prices and selling prices.
        """
        pressure_indicator = PriceAnalyser._get_pressure_indicator(
            open_price, close_price
        )
        normalized_volume = PriceAnalyser.get_normalized_series(volume, 10)
        buying_normalized_volume = (
            normalized_volume * (100 + pressure_indicator) / 200
        )
        selling_normalized_volume = (
            normalized_volume - buying_normalized_volume
        )
        buying_prices = PriceAnalyser.get_general_prices(
            close_price, buying_normalized_volume
        )
        selling_prices = PriceAnalyser.get_general_prices(
            close_price, selling_normalized_volume
        )
        return buying_prices, selling_prices



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="OhlcvAnalyser",
    version="1.0.1",
    packages=find_packages(),
    author="jackmappotion",
    author_email="jackmappotion@gmail.com",
    url="https://github.com/jackmappotion/OhlcvAnalyser",
    description="OhlcvAnalyser",
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from typing import List, Union
import pandas as pd
class VarianceAnalyser:
    @staticmethod
    def get_variance(series: pd.Series) -> float:
        """
        Calculate the variance of a series.

        Args:
            series (pd.Series): The input series.

        Returns:
            float: The variance of the series.
        """
        return series.var()

    @staticmethod
    def get_diff_series(a_series: pd.Series, b_series: pd.Series) -> pd.Series:
        """
        Calculate the difference series between two input series.

        Args:
            a_series (pd.Series): The first input series.
            b_series (pd.Series): The second input series.

        Returns:
            pd.Series: The difference series.
        """
        diff_series = a_series - b_series
        return diff_series

    @staticmethod
    def get_diff_variance(a_series: pd.Series, b_series: pd.Series) -> float:
        """
        Calculate the variance of the difference series between two input series.

        Args:
            a_series (pd.Series): The first input series.
            b_series (pd.Series): The second input series.

        Returns:
            float: The variance of the difference series.
        """
        diff_series = VarianceAnalyser.get_diff_series(a_series, b_series)
        diff_variance = VarianceAnalyser.get_variance(diff_series)
        return diff_variance

    @staticmethod
    def get_normalized_diff_series(a_series: pd.Series, b_series: pd.Series) -> pd.Series:
        """
        Calculate the normalized difference series between two input series.

        Args:
            a_series (pd.Series): The first input series.
            b_series (pd.Series): The second input series.

        Returns:
            pd.Series: The normalized difference series.
        """
        normalized_diff_series = (
            VarianceAnalyser.get_diff_series(a_series, b_series) / a_series
        )
        return normalized_diff_series

    @staticmethod
    def get_normalized_diff_variance(a_series: pd.Series, b_series: pd.Series) -> float:
        """
        Calculate the variance of the normalized difference series between two input series.

        Args:
            a_series (pd.Series): The first input series.
            b_series (pd.Series): The second input series.

        Returns:
            float: The variance of the normalized difference series.
        """
        normalized_diff_series = VarianceAnalyser.get_normalized_diff_series(
            a_series, b_series
        )
        normalized_variance = VarianceAnalyser.get_variance(
            normalized_diff_series
        )
        return normalized_variance